--[[--
@module MyDungeonsBook
]]

--[[--
UI
@section UI
]]

local L = LibStub("AceLocale-3.0"):GetLocale("MyDungeonsBook");

local function getAllDebuffsSpellMenu(rows, index, cols, challengeId)
    local challenge = MyDungeonsBook:Challenge_GetById(challengeId);
    local instanceId = challenge.challengeInfo.currentZoneId;
    local affixes = challenge.challengeInfo.affixes;
    local bosses = MyDungeonsBook:GetDungeonMetaInfo(instanceId).encounters;
    local spellId = rows[index].cols[1].value;
    local report = MyDungeonsBook:AllDebuffsFrame_Report_Create(rows[index], cols);
    return {
        MyDungeonsBook:WowHead_Menu_SpellComplex(spellId),
        MyDungeonsBook:Report_Menu(report),
        {
            text = L["Weak Auras"],
            hasArrow = true,
            menuList = {
                MyDungeonsBook:Wa_ContextMenuItem_Aura_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_PartyAura_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_PartyDispelAura_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_Fixate_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_GTFO_Create(spellId, instanceId, bosses, affixes),
            }
        }
    };
end

--[[--
Create a frame for Avoidable Debuffs tab (data is taken from `mechanics[**-AVOIDABLE-AURAS]`).

Mouse hover/out handler are included.

@param[type=Frame] parentFrame
@param[type=number] challengeId
@return[type=Frame] tableWrapper
]]
function MyDungeonsBook:AllDebuffsFrame_Create(parentFrame, challengeId)
    local avoidableDebuffsFrame = self:TabContentWrapperWidget_Create(parentFrame);
    local data = self:AllDebuffsFrame_GetDataForTable(challengeId, "PARTY-MEMBERS-AURAS");
    local columns = self:Table_Headers_GetForSpellsSummary(challengeId);
    tinsert(columns, 2, {
        name = " ",
        width = 40,
        align = "LEFT",
        DoCellUpdate = function(...)
            self:Table_Cell_FormatAsDispelIcon(...);
        end
    });
    columns[4].width = 100;
    local table = self:TableWidget_Create(columns, 12, 40, nil, avoidableDebuffsFrame, "all-debuffs");
    table:SetData(data);
    table:RegisterEvents({
        OnClick = function(_, _, data, _, _, realrow, column, _, button)
            if (button == "RightButton" and realrow) then
                EasyMenu(getAllDebuffsSpellMenu(data, realrow, table.cols, challengeId), self.menuFrame, "cursor", 0 , 0, "MENU");
            end
            self:Table_Cell_ShiftClick_Check(data, realrow, column, button, 3);
        end,
        OnEnter = function (_, cellFrame, data, _, _, realrow, column)
            if (realrow) then
                if (column == 2) then
                    self:Table_Cell_SpellDebuffMouseHover(cellFrame, data[realrow].cols[1].value);
                end
                if (column == 3 or column == 4) then
                    self:Table_Cell_SpellMouseHover(cellFrame, data[realrow].cols[1].value);
                end
            end
        end,
        OnLeave = function (_, _, _, _, _, realrow, column)
            if (realrow) then
                if (column == 2 or column == 3 or column == 4) then
                    self:Table_Cell_MouseOut();
                end
            end
        end
    });
    return avoidableDebuffsFrame;
end

--[[--
Map data about Avoidable Debuffs for challenge with id `challengeId`.

@param[type=number] challengeId
@param[type=string] key for mechanics table (it's different for BFA and SL)
@return[type=table]
]]
function MyDungeonsBook:AllDebuffsFrame_GetDataForTable(challengeId, key)
    local tableData = {};
    if (not challengeId) then
        return tableData;
    end
    local mechanics = self:Challenge_Mechanic_GetById(challengeId, key);
    if (not mechanics) then
        self:DebugPrint(string.format("No Debuffs data for challenge #%s", challengeId));
        return tableData;
    end
    for unitName, auras in pairs(mechanics) do
        for spellId, spellInfo in pairs(auras) do
            if (self.db.global.meta.spells[spellId] and self.db.global.meta.spells[spellId].auraType == "DEBUFF") then
                if (not tableData[spellId]) then
                    tableData[spellId] = {
                        spellId = spellId,
                        player = 0,
                        party1 = 0,
                        party2 = 0,
                        party3 = 0,
                        party4 = 0
                    };
                end
                local unitId = self:GetPartyUnitByName(challengeId, unitName);
                if (unitId) then
                    tableData[spellId][unitId] = spellInfo.meta.hits;
                end
            end
        end
    end
    local remappedTableData = {};
    for _, row in pairs(tableData) do
        local r = {
            cols = {}
        };
        tinsert(r.cols, {value = row.spellId});
        tinsert(r.cols, {value = row.spellId});
        tinsert(r.cols, {value = row.spellId});
        tinsert(r.cols, {value = row.spellId});
        local sum = 0;
        for _, unitId in pairs(self:GetPartyRoster()) do
            local isAvoidable = self:IsSpellAvoidableForPartyMember(challengeId, unitId, row.spellId);
            local cell = {value = row[unitId]};
            if (isAvoidable) then
                cell.color = {
                    r = 200,
                    g = 0,
                    b = 0,
                    a = 1
                };
            end
            tinsert(r.cols, cell);
            if (row[unitId]) then
                sum = sum + row[unitId];
            end
        end
        tinsert(r.cols, {value = sum});
        tinsert(remappedTableData, r);
    end
    return remappedTableData;
end

--[[--
@param[type=table] row
@param[type=table] cols
@return[type=table]
]]
function MyDungeonsBook:AllDebuffsFrame_Report_Create(row, cols)
    local spellLink = GetSpellLink(row.cols[1].value);
    local title = string.format(L["MyDungeonsBook Debuff %s hits:"], spellLink);
    return self:Table_PlayersRow_Report_Create(row, cols, {5, 6, 7, 8, 9, 10}, title);
end
